#!/bin/bash

ARGS="$*"
CONF=''
BLASTX="--protein_similarity_search_method miniprot_blastx"
SEQUENCES=''
MD5CONF=''
MD5SEQUENCES=''
SEQNAME=''
TAG=''

SCRIPT_DIR="$( cd -- "$( dirname -- "${BASH_SOURCE[0]:-$0}"; )" &> /dev/null && realpath . 2> /dev/null; )";

PURPLE='\033[0;35m'
NC='\033[0m'
TITLE='\033[1;91m'
PARAMS=`grep -h '=' $SCRIPT_DIR/conf/params/system.config | sed -E 's/[ \t]+//' | grep -v '^/'`
ERROR='\033[0;31m'
INFO='\033[0;32m'

HELPMESSAGE="
${TITLE}Usage:${NC}
	${INFO} $0 [--parameters] [--advanced] [--training_use_helixer] [--usearch] [--diamond] [--blastx] [--fg] [--tag] --sequences <path to fasta file> --config <path to EuGene-P config file (protein and ESTs dbs)>${NC}
	
${TITLE}Flags:${NC}
${INFO}
	--parameters : print basic parameters
	--advanced: print complete list of parameters
	
	--training_use_helixer: use Helixer (CPU mode) for training steps instead of blastx 
	
	--usearch: use usearch instead of miniprot
	--diamond: use diamond instead of miniprot
	--blastx: use diamond instead of miniprot
	--fg: launch main script in foreground (mandatory to be slurm submission compliant)
	--sequences: path to genome fasta file
	--config: path to genome config file
	
	--tag: add a tag to output directory name (only '_' or alphanumeric)
${NC}
"

if [ $# -lt 1 ]; then
  echo -e "$HELPMESSAGE"
  exit 0
fi


while [ $# -gt 0 ]; do

   if [[ $1 == *"--help"* ]]; then
        echo -e "$HELPMESSAGE"
        exit 0
   elif [[ $1 == *"--parameters"* ]]; then
        nextflow run ./egn-ep.nf $BLASTX --help
        exit 0
   elif [[ $1 == *"--advanced"* ]]; then
        echo -e "${TITLE}[ADVANCED PARAMETERS]${NC}\n\n${PURPLE}${PARAMS}${NC}" | less -r
        exit 0
   elif [[ $1 == *"--config"* ]]; then
        CONF="-c `realpath $2`"
        MD5CONF=`md5sum $2 | cut -d ' ' -f1`
   elif [[ $1 == *"--sequences"* ]]; then
        SEQUENCES=`realpath $2`
        MD5SEQUENCES=`md5sum $2 | cut -d ' ' -f1`
        SEQNAME=`basename $2 | rev | cut -d. -f2- | rev`
   elif [[ $1 == *"--usearch"* ]]; then
        BLASTX="--protein_similarity_search_method ublast_blastx"
   elif [[ $1 == *"--diamond"* ]]; then
        BLASTX="--protein_similarity_search_method diamond_blastx"
   elif [[ $1 == *"--blastx"* ]]; then
        BLASTX="--protein_similarity_search_method blastx"
   elif [[ $1 == *"--tag"* ]]; then
		pattern="[^a-zA-Z0-9_]"
		if ! [[ $2 =~ $pattern ]]; then
			TAG="-$2"
		else
			echo -e "${ERROR}[ERROR] --tag parameter must not contain space or special characters (only '_' or alphanumeric)${NC}"
			exit 1
		fi
   elif [[ $1 == *"--slurm"* || $1 == *"--fg"* ]]; then
        FG=true
   fi

  shift
done


if [[ $SEQUENCES == '' ]]; then
  echo -e "${ERROR}[ERROR] --sequences parameter is missing${NC}"
  exit 1
elif [[ $CONF == '' ]]; then
  echo -e "${ERROR}[ERROR] --config parameter is missing${NC}"
  exit 2
else
	
  DOTTEST=`grep '^>' $SEQUENCES | cut -f1 -d ' ' | grep '\.'`
  if [[ $DOTTEST != '' ]]; then
	echo -e "${ERROR}[ERROR] Sequence IDs contains '.' character - Please remove them before:\n$DOTTEST${NC}"
	exit 4
  fi
  
  UNIQTEST=`grep '^>' $SEQUENCES | cut -f1 -d ' ' | sort | uniq -c | grep -v ' 1 '`
  if [[ $UNIQTEST != '' ]]; then
	echo -e "${ERROR}[ERROR] Sequence IDs must be unique - Please fix them before:\n$UNIQTEST${NC}"
	exit 5
  fi
  
  RUNDIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )/" >/dev/null 2>&1 && realpath . )"
  DATE=`date +%Y%m%d-%s`
  RUNID=`echo "$MD5SEQUENCES" | cut -d ' ' -f1| sed -E 's/\w{24}$//'`
  RESULTID=`echo "$MD5SEQUENCES-$MD5CONF" | cut -d ' ' -f1| sed -E 's/\w{24}$//'`
  LOCKED=`find  $RUNDIR/runs/$SEQNAME-$RUNID/.nextflow/cache/ -name LOCK -exec lsof {}  2>/dev/null \;`
  
  
  
  if [[ $LOCKED == '' ]]; then
     export NXF_SINGULARITY_CACHEDIR=$RUNDIR/runs/$SEQNAME-$RUNID/singularity
     mkdir -p $RUNDIR/runs/$SEQNAME-$RUNID/singularity
     mkdir -p $RUNDIR/runs/$SEQNAME-$RUNID/logs
     rm -f $RUNDIR/runs/$SEQNAME-$RUNID/logs/latest.*
     echo -e "${PURPLE}\n[RUNID]\t >>>>  $SEQNAME-$RUNID  <<<< ${NC}\n\n"
     echo -e "${INFO}[INFO] Launch:  ${NC}\n\tnextflow run $RUNDIR/egn-ep.nf  -resume $ARGS $CONF $BLASTX\n\n"
     echo -e "${INFO}[INFO] Results directory\truns/$SEQNAME-$RUNID/results-$DATE$TAG${NC}"
     echo -e "${INFO}[INFO] Logs    directory\truns/$SEQNAME-$RUNID/logs${NC}"
     echo -e "${INFO}[INFO] Work    directory\truns/$SEQNAME-$RUNID/work${NC}"
     echo -e "${INFO}[INFO] Apps    directory\truns/$SEQNAME-$RUNID/singularity${NC}"
     cd $RUNDIR/runs/$SEQNAME-$RUNID/logs/
     ln -s $DATE$TAG.nextflow.log latest.nextflow.log
     ln -s $DATE$TAG.stdout latest.stdout
     ln -s $DATE$TAG.stderr latest.stderr
     cd $RUNDIR/runs/$SEQNAME-$RUNID/
     mkdir results-$DATE$TAG
     rm -f latest && ln -s results-$DATE$TAG latest
     if [[ ! $FG == '' ]]; then
          echo -e "${INFO}[INFO] Submitted in foreground mode${NC}"
          (cd $RUNDIR/runs/$SEQNAME-$RUNID/ && nextflow \
          -log $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE$TAG.nextflow.log \
          run $RUNDIR/egn-ep.nf -ansi-log false -resume $ARGS $CONF $BLASTX \
          -work-dir $RUNDIR/runs/$SEQNAME-$RUNID/work \
          --outdir $RUNDIR/runs/$SEQNAME-$RUNID/results-$DATE$TAG \
          > $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE$TAG.stdout ) 2> $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE$TAG.stderr
     else
          echo -e "${INFO}[INFO] Run in bash mode (nohup and background)${NC}"
          (cd $RUNDIR/runs/$SEQNAME-$RUNID/ && nohup nextflow \
          -log $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE$TAG.nextflow.log \
          run $RUNDIR/egn-ep.nf -ansi-log false -resume $ARGS $CONF $BLASTX \
          -work-dir $RUNDIR/runs/$SEQNAME-$RUNID/work \
          --outdir $RUNDIR/runs/$SEQNAME-$RUNID/results-$DATE$TAG \
          > $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE$TAG.stdout ) 2> $RUNDIR/runs/$SEQNAME-$RUNID/logs/$DATE.stderr&
     echo -e "${NC}[TIP] Kill Main Process\t\$ ./utils/killRun.sh $SEQNAME-$RUNID${NC}"
     
     fi
     cd $RUNDIR
     echo -e "${NC}[TIP] Follow execution\t\$ tail -f ./runs/$SEQNAME-$RUNID/logs/$DATE$TAG.stdout${NC}"
     echo -e "${NC}[TIP] Failed Tasks\t\$ ./utils/failedTasks.sh $SEQNAME-$RUNID${NC}"
     echo -e "${NC}[TIP] Count Tasks\t\$ ./utils/countTasks.sh $SEQNAME-$RUNID${NC}"
     

  else
     echo -e "${ERROR}[ERROR] There's an already running job for this sequence${NC}"
     echo -e "$LOCKED"
     exit 3
  fi
fi
