nextflow.enable.dsl=2

process LIPMUTILS_download {

	publishDir("${params.outdir}/download", mode:'copy')
	storeDir("${params.store}")
	
	tag("Download ${url} to ${outfile}.${suffix}.${ext}")
	time '24h'
	
	input:
	val(url)
	val(outfile)
	val(suffix)
	val(ext)
	
	output:
	path("${outfile}.${suffix}.${ext}")
	
	script:
	"""
	wget -O ${outfile}.${suffix}.${ext} '$url'
	"""
}

process LIPMUTILS_fixIds {

	input:
	path(infile)
	val(outfile)
	val(suffix)
	val(ext)

	output:
	path("${outfile}.${suffix}.${ext}")
	
	script:
	"""
	lipm_nr_rename.pl --in  ${infile} --out ${outfile}.${suffix}.${ext}
	"""
}

process LIPMUTILS_getUniprotRelease {

	publishDir("${params.outdir}/download", mode:'copy')
	cache false	
	
	output:
	stdout
	
	
	script:
	"""
	wget -O - '${params.uniprot_ftp_url}/taxonomic_divisions/reldate.txt' > reldate.txt
	head -1 reldate.txt | perl -ne 'print /(\\d{4}_\\d{2})/'
	"""
}



process EGNPCORE_dfam2fasta {
	
	input:
	path(embl)
	
	output:
	path("Dfam-${params.dfam_version}.consensus.fasta")

	script:
	"""
	convert_dfam_consensus.pl --embl ${embl} --outfasta Dfam-${params.dfam_version}.consensus.fasta
	"""
}

process EGNPCORE_uniprot2fasta {
	
	input:
	path(embl)
	val(pe)
	val(minlen)
	
	output:
	path("${embl.baseName}.pe${pe}.minlen${minlen}.fasta")

	script:
	"""
	filter_uniprot_dat.pl  --dat ${embl} --pe ${pe} --min ${minlen} --outfasta ${embl.baseName}.pe${pe}.minlen${minlen}.fasta
	"""
}

process LIPMUTILS_idxRepeat {

	input:
	path(fasta)
	val(dbtype)
	
	output:
	path("${fasta}.*")
	
	
	script:
	"""
	makeblastdb -parse_seqids -dbtype ${dbtype} -in ${fasta}
	"""
}

process USEARCH_blastRepeat {

	label "process_high"
	
	input:
	path(fasta)
	path(dfam)
	path(dfam_idx)
	path(transposonpsi)
	path(transposonpsi_idx)
	val(blast_params)
	
	output:
	path("${fasta}.m8")
	
	script:
	
	"""
	touch ${fasta}.m8
	usearch -threads 12  -ublast  ${dfam} -db  ${fasta} -evalue 1e-3 -accel 1.0 -blast6out ${fasta}.reduce.m8
	cut -f1  ${fasta}.reduce.m8 | sort -u > dfam.reduce && usearch -fastx_getseqs ${dfam} -labels dfam.reduce  -fastaout ${dfam}.reduce 
	cut -f2  ${fasta}.reduce.m8 | sort -u > fasta.reduce && usearch -fastx_getseqs ${fasta} -labels fasta.reduce  -fastaout ${fasta}.reduce 
	echo "makeblastdb"
	makeblastdb -dbtype nucl -in  ${dfam}.reduce  -logfile dfam.reduce.log
	echo "blastdbcmd"
	blastdbcmd -info -db ${dfam} -logfile dfam.log > dfam.info
	echo "parse"
	head -2 dfam.info | tail -n1 | cut -d ';' -f2 | cut -d ' ' -f2 | sed 's/,//g' > dfam.dbsize
	echo "tblastn"
	tblastn -query ${fasta}.reduce   -evalue 1E-10 -max_target_seqs 1 -dbsize `cat dfam.dbsize`  -outfmt '6 std qlen slen qcovs stitle' -num_threads 12 -db ${dfam}.reduce  >> ${fasta}.m8
	blastp -query ${fasta} ${blast_params} -outfmt '6 std qlen slen qcovs stitle'  -num_threads ${task.cpus} -db ${transposonpsi} >> ${fasta}.m8
	"""
}

process LIPMUTILS_filterBlast {
	input:
	path(m8)
	val(dbname)
	
	output:
	path("${dbname}.m8.fil")
	
	script:
	
	evalue = params.cleanprotdb_evalue
	qcovs = params.cleanprotdb_qcovs
	identity = params.cleanprotdb_identity
	
	"""
	cat ${m8} | perl -ne '\$l = \$_; chomp \$l;@a = split ("\\t", \$l); print "\$l\n" if (\$a[2] >= ${identity} && \$a[10] <= ${evalue} && \$a[14] >= ${qcovs})' > ${dbname}.m8.fil
	"""
	
}


process LIPMUTILS_filterProtDB {
    
    label 'process_high'
    
    input:
    val(dbname)
    path(db)
    path(m8fil)
    
    
    output:
    path(cleandb)
    
    script:

    cleandb = dbname + ".noTE.fasta"
        
    """
    lipm_fastafilter.pl --in ${db} --out ${cleandb} --db_filename ${m8fil} --db_column 1 --remove
	"""
    
}



process CDHIT_compress {

    label 'process_high'
     
    input:
    val(name)
    path(infile)
    val(memory)
    
	output:
    path("${name}.cdhit.fasta")
    
    script:
    """
    cd-hit -M ${memory} -T ${task.cpus} ${params.cdhit_params} -i ${infile} -o ${name}.cdhit.fasta
    """
}
