#!/bin/bash


INFO='\033[0;32m'
NC='\033[0m'
ERROR='\033[0;31m'

ROOT_CONTAINER=https://lipm-browsers.toulouse.inra.fr/pub/singularity-repository/egnep/current/
MINIPROT_CONTAINER=blast-2.14.0_miniprot-0.12.sif
LIPMUTILS_CONTAINER=lipmutils-1.0.sif
MINIPROT_PARAMS=' -F 50 '
GENOME=""
PROTEOME=""
OUTDIR=""
PREFIX=""
CPUS=2
USAGE=" --genome  <PATH TO GENOME (fasta)> --proteome <PATH TO PROTEOME FILE (fasta)> --prefix <output prefix file> --cpus <CPUS> --outdir <PATH TO OUTPUT DIRECTORY>"

if [ $# -lt 5 ]; then
  echo -e "\nUsage:\n$0 $USAGE\n"
  exit 0
fi


while [ $# -gt 0 ]; do

   if [[ $1 == *"--help"* ]]; then
        echo -e "\nUsage:\n$0 $USAGE\n"
        exit 0
   elif [[ $1 == *"--genome"* ]]; then
        GENOME=`realpath $2`
   elif [[ $1 == *"--proteome"* ]]; then
        PROTEOME=`realpath $2`
   elif [[ $1 == *"--prefix"* ]]; then
        PREFIX=$2
   elif [[ $1 == *"--cpus"* ]]; then
        CPUS=$2
   elif [[ $1 == *"--outdir"* ]]; then
        OUTDIR=`realpath $2`
   fi

  shift
done



if [ ! -f "$GENOME" ]; then
	echo -e "${ERROR}ERROR - genome >$GENOME<not found${NC}"
	exit 1
fi

if [ ! -f "$PROTEOME" ]; then
	echo -e "${ERROR}ERROR - proteome >$PROTEOME<not found${NC}"
	exit 1
fi

re='^[0-9]+$'
if ! [[ $CPUS =~ $re ]] ; then
   echo -e "${ERROR}ERROR - cpus >$CPUS< not valid - Integer required${NC}"
   exit 1
fi

reprefix='^[A-Za-z]+$'
if ! [[ $PREFIX =~ $reprefix ]] ; then
   echo -e "${ERROR}ERROR - prefix >$PREFIX< not valid - Letters only required${NC}"
   exit 1
fi

mkdir -p $OUTDIR

if [ ! -e $MINIPROT_CONTAINER ]; then
	echo -e "${INFO}[INFO] Retrieve Miniprot Container:  ${NC}\n\twget $ROOT_CONTAINER$MINIPROT_CONTAINER\n"
	wget $ROOT_CONTAINER$MINIPROT_CONTAINER
fi

if [ ! -e $LIPMUTILS_CONTAINER ]; then
	echo -e "${INFO}[INFO] Retrieve LipmUtils Container:  ${NC}\n\twget $ROOT_CONTAINER$LIPMUTILS_CONTAINER\n"
	wget $ROOT_CONTAINER$LIPMUTILS_CONTAINER
fi

chmod +x $MINIPROT_CONTAINER
chmod +x $LIPMUTILS_CONTAINER

GENOME_DIR=`dirname $GENOME`
PROTEOME_DIR=`dirname $PROTEOME`


CMD=""
if [ ! -e  $OUTDIR/$PREFIX.miniprot.raw.gff ]; then
	CMD="miniprot $GENOME $PROTEOME -P $PREFIX $MINIPROT_PARAMS -t $CPUS --gff"
	echo -e "${INFO}[INFO] Run Miniprot:  ${NC}\n\tsingularity exec -B $GENOME_DIR -B $PROTEOME_DIR ./$MINIPROT_CONTAINER $CMD\n"
	singularity exec -B $OUTDIR -B $GENOME_DIR -B $PROTEOME_DIR ./$MINIPROT_CONTAINER $CMD  > $OUTDIR/$PREFIX.miniprot.raw.gff
fi

if [ ! -e  $PROTEOME.fai ]; then
	CMD="samtools faidx $PROTEOME"
	echo -e "${INFO}[INFO] Run Samtools:  ${NC}\n\tsingularity exec -B $PROTEOME_DIR ./$LIPMUTILS_CONTAINER $CMD\n"
	singularity exec -B $OUTDIR -B $PROTEOME_DIR ./$LIPMUTILS_CONTAINER $CMD
fi

if [ ! -e  $OUTDIR/$PREFIX.miniprot.filter.gff ]; then
	CMD="lipm_filter_miniprot_gff.pl --fai $PROTEOME.fai --gffin $OUTDIR/$PREFIX.miniprot.raw.gff  --gffout $OUTDIR/$PREFIX.miniprot.filter.gff --pcs 50 --noframeshift --nostop"
	echo -e "${INFO}[INFO] Run Miniprot Filter:  ${NC}\n\tsingularity exec -B $GENOME_DIR -B $PROTEOME_DIR ./$LIPMUTILS_CONTAINER $CMD\n"
	singularity exec -B $OUTDIR -B $GENOME_DIR -B $PROTEOME_DIR ./$LIPMUTILS_CONTAINER $CMD
fi

if [ ! -e  $OUTDIR/$PREFIX.miniprot.filter.fasta ]; then
	CMD="gffread  -w $OUTDIR/$PREFIX.miniprot.filter.fasta -g $GENOME $OUTDIR/$PREFIX.miniprot.filter.gff"
	echo -e "${INFO}[INFO] Run Gffread:  ${NC}\n\tsingularity exec -B $OUTDIR -B $GENOME_DIR ./$LIPMUTILS_CONTAINER $CMD\n"
	singularity exec -B $OUTDIR -B $GENOME_DIR ./$LIPMUTILS_CONTAINER $CMD
fi


CMD="lipm_nrdb -o $OUTDIR/$PREFIX.transcripts.fasta $OUTDIR/$PREFIX.miniprot.filter.fasta "
echo -e "${INFO}[INFO] Remove redundancy:  ${NC}\n\tsingularity exec -B $OUTDIR ./$LIPMUTILS_CONTAINER $CMD\n"
singularity exec -B $OUTDIR ./$LIPMUTILS_CONTAINER $CMD

echo -e "${INFO}[INFO] Cleaned file :  >>> $OUTDIR/$PREFIX.transcripts.fasta\t <<< ${NC}"


exit 0
